import { of } from 'rxjs';
import {
  tap,
  switchMap,
  map,
  catchError,
  exhaustMap,
  debounceTime
} from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';

import { Store } from '@ngrx/store';
import { Actions, createEffect, ofType } from '@ngrx/effects';

import { EventProtocolService } from '@ucap/ng-protocol-event';
import { RoomProtocolService } from '@ucap/ng-protocol-room';
import { FileProtocolService } from '@ucap/ng-protocol-file';

import * as roomActions from '../room/actions';
import {
  events,
  eventsFailure,
  eventsSuccess,
  read,
  readFailure,
  readSuccess,
  fileInfos,
  fileInfosFailure,
  fileInfosSuccess
} from './actions';

import { InfoRequest, ReadResponse, FileType } from '@ucap/protocol-event';

import { ModuleConfig } from '../../config/module-config';
import { _MODULE_CONFIG } from '../../config/token';

@Injectable()
export class Effects {
  selectedRoom$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(roomActions.selectedRoom),
        debounceTime(300),
        tap((action) => {
          const requestCount = this.moduleConfig?.eventRequestInitCount || 50;
          const req: InfoRequest = {
            roomId: action.roomId,
            baseSeq: 0,
            requestCount
          };

          this.store.dispatch(events({ req }));
        })
      );
    },
    { dispatch: false }
  );

  events$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(events),
        map((action) => action.req),
        switchMap((req) => {
          return this.eventProtocolService.info(req).pipe(
            map((res) => {
              if (!!res && !!res.res) {
                const infoList = res.infoList;

                this.store.dispatch(
                  eventsSuccess({
                    eventInfoList: infoList,
                    res: res.res,
                    remainEvent:
                      infoList.length === req.requestCount ? true : false
                  })
                );

                if (req.baseSeq === 0) {
                  // 최초 이벤트 목록 조회.

                  // SSVC_TYPE_EVENT_READ_REQ 수행.
                  const maxSeq = Math.max.apply(
                    Math,
                    infoList.map((v) => v.seq)
                  );
                  this.store.dispatch(
                    read({
                      roomId: req.roomId,
                      lastReadSeq: Number(maxSeq)
                    })
                  );

                  // File 정보 수집.
                  this.store.dispatch(
                    fileInfos({
                      req: {
                        roomId: res.res.roomId,
                        // { 파일타입 } cf) I : 이미지 V: 동영상 F: 파일 "" 빈값이면 모든 타입을 내려줌
                        type: FileType.All
                      }
                    })
                  );
                } else {
                }
              }
            }),
            catchError((error) =>
              of(eventsFailure({ roomId: req.roomId, error }))
            )
          );
        })
      );
    },
    { dispatch: false }
  );

  read$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(read),
        exhaustMap((req) =>
          this.eventProtocolService.read(req).pipe(
            map((res: ReadResponse) => {
              this.store.dispatch(readSuccess(res));
            }),
            catchError((error) => of(readFailure({ error })))
          )
        )
      );
    },
    { dispatch: false }
  );

  fileInfos$ = createEffect(
    () => {
      return this.actions$.pipe(
        ofType(fileInfos),
        switchMap((action) => {
          return this.fileProtocolService.info(action.req).pipe(
            map((res) => {
              this.store.dispatch(
                fileInfosSuccess({
                  fileInfoList: res.fileInfos,
                  fileInfoCheckList: res.fileInfoChecks,
                  res: res.res
                })
              );
            }),
            catchError((error) =>
              of(fileInfosFailure({ roomId: action.req.roomId, error }))
            )
          );
        })
      );
    },
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    @Inject(_MODULE_CONFIG) private moduleConfig: ModuleConfig,
    private roomProtocolService: RoomProtocolService,
    private eventProtocolService: EventProtocolService,
    private fileProtocolService: FileProtocolService
  ) {}
}
